package sun.security.krb5.internal.rcache;

import java.util.Iterator;
import java.util.ListIterator;
import sun.security.krb5.internal.KrbApErrException;
import sun.security.krb5.internal.KerberosTime;
import java.util.LinkedList;

public class AuthList
{
    private final LinkedList<AuthTimeWithHash> entries;
    private final int lifespan;
    private volatile int oldestTime;
    
    public AuthList(int lifespan) {
        this.oldestTime = Integer.MIN_VALUE;
        this.lifespan = lifespan;
        this.entries = new LinkedList();
    }
    
    public synchronized void put(AuthTimeWithHash authTimeWithHash, KerberosTime kerberosTime) throws KrbApErrException {
        if (this.entries.isEmpty()) {
            this.entries.addFirst(authTimeWithHash);
            this.oldestTime = authTimeWithHash.ctime;
            return;
        }
        int compareTo = ((AuthTimeWithHash)this.entries.getFirst()).compareTo(authTimeWithHash);
        if (compareTo < 0) {
            this.entries.addFirst(authTimeWithHash);
        }
        else {
            if (compareTo == 0) {
                throw new KrbApErrException(34);
            }
            ListIterator<AuthTimeWithHash> listIterator = this.entries.listIterator(1);
            boolean n = false;
            while (listIterator.hasNext()) {
                AuthTimeWithHash o = (AuthTimeWithHash)listIterator.next();
                int compareTo2 = o.compareTo(authTimeWithHash);
                if (compareTo2 < 0) {
                    this.entries.add(this.entries.indexOf(o), authTimeWithHash);
                    n = true;
                    break;
                }
                if (compareTo2 != 0) {
                    continue;
                }
                throw new KrbApErrException(34);
            }
            if (n == 0) {
                this.entries.addLast(authTimeWithHash);
            }
        }
        long n2 = (long)(kerberosTime.getSeconds() - this.lifespan);
        if ((long)this.oldestTime > n2 - 5L) {
            return;
        }
        while (!this.entries.isEmpty()) {
            AuthTimeWithHash e = (AuthTimeWithHash)this.entries.removeLast();
            if ((long)e.ctime >= n2) {
                this.entries.addLast(e);
                this.oldestTime = e.ctime;
                return;
            }
            continue;
        }
        this.oldestTime = Integer.MIN_VALUE;
        return;
    }
    
    public boolean isEmpty() {
        return this.entries.isEmpty();
    }
    
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        Iterator<AuthTimeWithHash> descendingIterator = this.entries.descendingIterator();
        int size = this.entries.size();
        while (descendingIterator.hasNext()) {
            sb.append('#').append(size--).append(": ").append(((AuthTimeWithHash)descendingIterator.next()).toString()).append('\n');
        }
        return sb.toString();
    }
}
